var html_at = require('../../ast').html;
var at = require('../../ast').js;
var path = require('path');
var l10nContext = require('../l10n/context.js');
var throwIdx = 0;

module.exports = function(flow){
  var fconsole = flow.console;
  var queue = flow.files.queue;

  //
  // Init js section
  //

  fconsole.log('Init js');

  var globalScope = new at.Scope('global');
  var globalToken = ['object', []];
  globalScope.put('global', '?', globalToken);
  globalScope.put('this', '?', globalToken);
  globalToken.obj = globalScope.names;
  //globalScope.put('__resources__', 'hz', ['object', []]);

  flow.js = {
    globalScope: globalScope,
    rootBaseURI: {},
    rootNSFile: {},
    getFileContext: getFileContext,
    //classes: [],
    resources: [],
    namespaces: {},
    fn2ns: {}
    //resources: {}
  };

  flow.tmpl = {
    themes: {},
    themeResources: {}
  };

  flow.l10n = {
    cultureList: [],  // TODO: fetch culture list from basis.l10n
    defList: [],
    getTokenList: [],
    pathes: {},
    tokens: {},

    dictList: {}
  };


  //
  // Process files
  //

  fconsole.start('Process scripts');
  for (var i = 0, file; file = queue[i]; i++)
    if (file.type == 'script' && !file.ast)
    {
      fconsole.start(file.relpath);

      if (file.htmlNode)
      {
        var attrs = html_at.getAttrs(file.htmlNode);
        var configAttr = false;

        if (attrs.hasOwnProperty('data-basis-config'))
          configAttr = 'data-basis-config';
        else
          if (attrs.hasOwnProperty('basis-config'))
            configAttr = 'basis-config';

        if (configAttr)
        {
          fconsole.log('[i] basis.js marker found (' + configAttr + ' attribute)');
          processBasisFile(flow, file, attrs[configAttr] || '');
        }
      }

      processFile(file, flow);

      fconsole.endl();
    }
  fconsole.endl();
};

module.exports.handlerName = '[js] Extract';


//
// main part
//

function getFileContext(file){
  return {
    __filename: file.filename || '',
    __dirname: file.baseURI,
    namespace: file.namespace || ''
  };
}

function createScope(file, flow, module){
  if (file.type == 'script' && !file.jsScope)
  {
    var exports;

    if (!module)
    {
      exports = ['object', []];
      module = ['object', [
        ['exports', exports]
      ]];
      module.obj = {
        exports: exports
      };
    }
    else
    {
      exports = module.obj.exports;
    }

    var scope = new at.Scope('function', flow.js.globalScope, exports);
    var basisResource = flow.js.globalScope.resolve(['dot', ['name', 'basis'], 'resource']);
    var basis = flow.js.globalScope.resolve(['name', 'basis']);
    var names = {
      __filename: ['string', '/' + file.relpath],
      __dirname: ['string', flow.js.oldBasis
        ? file.baseURI
        : file.baseURI.replace(/\/$/, '')
      ],
      global: flow.js.globalScope.resolve(['name', 'global']),
      basis: flow.js.globalScope.resolve(['name', 'basis']),
      resource: at.createRunner(function(token, this_, args){
        // resource(path) -> basis.resource("__dirname" + path)
        token[1] = ['dot', ['name', 'basis'], 'resource'];
        token[1].ref_ = basisResource;
        token[1].refPath_ = 'basis.resource';
        token[2] = [args[0] = ['binary', '+', ['string', file.baseURI], args[0]]];
        basisResource.run.call(this, token, this_, args);
      }),
      module: module,
      exports: exports
    };

    if (!flow.js.oldBasis)
    {
      var basisRequire = flow.js.globalScope.resolve(['dot', ['name', 'basis'], 'require']);
      names.require = at.createRunner(function(token, this_, args){
        // require(path, base) -> basis.require(path, base || "__dirname")
        token[1] = ['dot', ['name', 'basis'], 'require'];
        token[1].ref_ = basisRequire;
        token[1].refPath_ = 'basis.require';
        token[2] = [args[0], args[1] || (args[1] = names.__dirname.slice(0))];
        basisRequire.run.call(this, token, this_, args);
      });
    }

    for (var name in names)
      scope.put(name, 'arg', names[name]);

    file.jsScope = scope;
  }
}

function defineHandler(scope, name, fn){
  if (name.indexOf('.') != -1)
  {
    var token = scope.resolve(at.parse(name, 1));
    
    if (!token)
      throw 'handler ' + name + ' is not resolved in specified scope';

    token.run = fn;
  }
  else
  {
    var symbol = scope.get(name);

    if (!symbol)
      throw 'symbol ' + name + ' is not resolved in specified scope';

    symbol.run = fn;  // does it affect anything?
    symbol.token.run = fn;
    symbol.token = at.createRunner(fn);
  }
}

function astExtend(scope, dest, source){
  if (dest && source && source[0] == 'object')
  {
    if (!dest.obj)
      dest.obj = {};
    if (!dest.objSource)
      dest.objSource = {};
    for (var i = 0, props = source[1], prop; prop = props[i]; i++)
    {
      dest.obj[prop[0]] = scope.resolve(prop[1]) || prop[1];
      dest.objSource[prop[0]] = source;
    }
  }
}

function astComplete(scope, dest, source){
  if (dest && source && source[0] == 'object')
  {
    if (!dest.obj)
      dest.obj = {};
    if (!dest.objSource)
      dest.objSource = {};    
    for (var i = 0, props = source[1], prop; prop = props[i]; i++)
      if (prop[0] in dest.obj == false)
      {
        dest.obj[prop[0]] = scope.resolve(prop[1]) || prop[1];
        dest.objSource[prop[0]] = source;
      }
  }
}

function processFile(file, flow){
  var defineHandler_ = defineHandler;
  defineHandler = function(scope, name, fn){
    try {
      defineHandler_(scope, name, fn);
    } catch(e) {
      flow.warn({
        file: file.relpath,
        message: e
      });
    }
  }


  // if file has ast - it's already processed
  if (file.ast)
    return;

  var globalScope = flow.js.globalScope;
  var fconsole = flow.console;
  var content = file.content;

  if (flow.options.jsCutDev)
    content = content.replace(/(;;;|\/\*\*\s*@cut.*?\*\/).*([\r\n]|$)/g, '$2');

  // extend file info
  file.context = getFileContext(file);
  file.deps = [];
  file.resources = [];

  // parse
  try {
    file.ast = at.parse(content);
  } catch(e) {
    file.ast = ['toplevel', []];
    flow.warn({
      fatal: true,
      file: file.relpath,
      message: 'Javascript parse error of ' + file.relpath + (' (line: ' + e.line + ', col: ' + e.col + ')') + ':\n' + (e.message || e)
    });
  }

  // apply scope
  if (!file.jsScope)
    file.jsScope = globalScope;

  file.ast = at.applyScope(file.ast, file.jsScope);

  if (file.jsScope != globalScope)
  {
    file.ast[1] = [['function', null, ['exports', 'module', 'basis', 'global', '__filename', '__dirname'],
      file.ast[1]
    ]];
    file.ast[1][0].scope = file.jsScope;
  }

  // collect throws
  file.throwCodes = file.ast.throws.map(function(token){
    return [++throwIdx, token.slice(), token];
  });

  try {
    switch (file.namespace)
    {
      case 'basis':
        fconsole.log('[i] load basis.js module and add to global scope');

        var keys = Object.keys; // FIXME: basis.js overload Object.keys and break standart output functions behaviour, become stable for this case
        global.basis = require(flow.js.basisScript).basis;
        global.basis.config.extProto = false;
        Object.keys = keys; // Object.keys overload fix  

        // omit dev output
        basis.dev.log = function(){};
        basis.dev.info = function(){};
        // override dev warn output
        basis.dev.warn = function(){
          flow.warn({
            message: ['basis.js warning:'].concat(Array.prototype.slice.call(arguments))
          });
        };
      break;

      case 'basis.template':
        fconsole.log('[i] load basis/template.js module');

        basis.require('basis.template');
      break;
    }
  } catch(e) {
    flow.warn({
      fatal: true,
      file: file.relpath,
      message: 'Namespace ' + file.namespace + ' load fault:\n' + (e.message || e)
    });
  }

  if (file.basisScript)
  {
    // get last global subscope as basis scope
    var basisScope = file.ast.scope.subscopes.slice(-1)[0];
    file.jsScope = basisScope;
    file.jsScope.put('global', 'arg', globalScope.get('global'));

    // flow.js.basisPackageClue = flow.files.add({
    //   inline: true,
    //   type: 'script',
    //   content: '[].forEach(' + function(module){
    //     var fn = module[1];
    //     var ns = basis.namespace(module[0]);
    //     // basis, module, exports, resource, global, __dirname, __filename
    //     //fn.call(ns, basis, ns, ns.exports, basis.resource, this, "", "");
    //     // 'exports', 'module', 'basis', 'global', '__filename', '__dirname', 'resource'
    //     fn.call(ns, ns.exports, ns, basis, this, "", "", basis.resource);
    //     basis.object.complete(ns, ns.exports);
    //   }.toString() + ', this)'
    // });


    //
    // namespaces
    //

    var createNS = function(path){
      var pathToken = ['string', path];
      var exports = ['object', [
        ['path', pathToken]
      ]];
      exports.obj = {
        path: pathToken
      };
      
      var token = ['function', path, []];
      token.objSource = {};
      token.obj = {
        extend: at.createRunner(function(token_, this_, args){
          astExtend(this.scope, this_.obj.exports, args[0]);
          astComplete(this.scope, this_, args[0]);
          token_.obj = this_.obj;
          token_.ref_ = token;
        }),
        path: ['string', path],
        exports: exports,
        setWrapper: at.createRunner(function(token_, this_, args){
          //fconsole.log('setWrapper', arguments);
          token.setWrapper_ = args[0];
          if (args[0].ref_)
          {
            //token.ref_ = args[0].ref_;
            //token.refPath_ = args[0].refPath_;
          }
        })
      };

      token.run = function(token_, this_, args){
        //fconsole.log('!ns call', token);
        if (token.setWrapper_)
        {
          if (token.setWrapper_.run)
            token.setWrapper_.run.call(this, token_, this_, args);
          else
          {
            // temporary commented, until handlers for wrap functions
            /*flow.warn({
              file: this.file.relpath,
              message: 'namespace ' + path + ' was called, but namespace wrapper has no run handler'
            });*/
          }
        }
        else
        {
          // temporary commented, until handlers for wrap functions
          /*flow.warn({
            file: this.file.relpath,
            message: 'namespace ' + path + ' was called, but namespace has no wrapper'
          });*/
        }
      }

      flow.js.namespaces[path] = token;

      return token;
    };    

    var getNamespace = function(namespace){
      var path = namespace.split('.');
      var root = path[0];
      var ns = globalScope.get(root);

      if (!ns)
      {
        ns = globalScope.put(root, 'ns', createNS(root)).token;
        ns.scope = globalScope;
      }
      else
        ns = ns.token;

      for (var i = 1; i < path.length; i++)
      {
        if (!ns.obj[path[i]])
          ns.obj[path[i]] = createNS(path.slice(0, i + 1).join('.'));

        ns = ns.obj[path[i]];
      }

      return ns;
    }; 


    //
    // resources
    //

    var resourceMap = {};
    var createResource = function(uri, resourceFile){
      if (resourceMap[uri])
        return resourceMap[uri];

      function createFetchMethod(methodName){
        return function(token){
           if (resourceFile.type == 'script') // && this.file.jsScope === this.scope)
           {
            // fconsole.log('[basis.resource]', methodName, '(' + resourceFile.relpath + ')', 'detected on top level -> add deps to contained file');
            fconsole.log('[basis.resource]', methodName, '(' + resourceFile.relpath + ')');
            if (!resourceFile.deps)
            {
              fconsole.incDeep(2);
              processFile(resourceFile, flow);
              fconsole.decDeep(2);
            }

            if (resourceFile.deps)
              resourceFile.deps.forEach(this.file.deps.add, this.file.deps);

            token.ref_ = resourceFile.jsScope.get('module').token.obj.exports;
          }
          // else
          // {
          //   fconsole.log('[basis.resource]', methodName, '(' + resourceFile.relpath + ')', 'detected, but not on top level - ignored');
          // }
        }
      }

      var token = resourceMap[uri] = ['function', null, []];
      token.obj = {
        fetch: at.createRunner(createFetchMethod('resource#fetch method call'))
      };
      token.run = createFetchMethod('resource call');
      return token;
    };

    function getResourceRunner(token, this_, args){
      // fconsole.log('basis.resource', token);

      if (!args[0])
      {
        //console.log('basis.resource', arguments);
        return;
      }

      // FIXME: temporary solution to avoid warnings in basis.template/basis.l10n that using
      // basis.resource to get templates & dictionary dinamically
      var flowArg = /^basis\.(template|l10n)$/.test(this.file.namespace) ? { warn: function(){} } : flow;

      args = token[2]; // FIXME: it's a hack
      var newFilename = args[0][0] == 'string'
        ? args[0][1]
        : at.getCallArgs(args, this.file.context, flowArg, this.file)[0];
      // var arg = this.scope.simpleExpression(args[0]);
      // var newFilename = arg && arg[0] == 'string' ? arg[1] : null;
      if (newFilename)
      {
        var file = this.file;
        var newFile = flow.files.add({
          filename: path.resolve(path.dirname(flow.options.file), newFilename),
          jsRefCount: 0
        });
        newFile.isResource = true;

        createScope(newFile, flow);

        file.link(newFile, token);
        file.resources.push(newFile);
        newFile.jsRefCount++;

        token.resourceRef = newFile;
        token[2] = [['string', newFile.relpath]];
        token.call = createResource(newFilename, newFile);
        token.obj = token.call.obj;

        flow.js.resources.push(token);
      }
    }


    //
    // main part
    //

    var handlers = {
      // basis.object.extend
      extend: function(token, this_, args){
        //fconsole.log('extend', arguments);
        if (this.file.jsScope == basisScope)
        {
          var arg0 = token[2][0];
          if (arg0[0] == 'name' && arg0[1] == 'Object')
            flow.js.oldBasis = true;
        }

        astExtend(this.scope, args[0], args[1]);
        token.obj = args[0];
      },

      // basis.object.complete
      complete: function(token, this_, args){
        //fconsole.log('comlete', arguments);
        astComplete(this.scope, args[0], args[1]);
        token.obj = args[0];
      },

      // basis.namespace
      getNamespace: function(token, this_, args){
        //fconsole.log('getNamespace', arguments);
        var namespace = args[0];
        if (namespace && namespace[0] == 'string')
        {
          token.obj = getNamespace(namespace[1]).obj;
          if (args[1])
            token.obj.setWrapper.run(token, this_, [args[1]]);
        }
      },

      // basis.require
      requireNamespace: function(token, this_, args){
        //fconsole.log('requireNamespace', token);
        if (args[0] && args[0][0] == 'string')
        {
          var arg0 = args[0][1];
          if (!/[^a-z0-9_\.]/i.test(arg0) && path.extname(arg0) != '.js')
          {
            var namespace = args[0][1];
            var parts = namespace.split(/\./);
            var root = parts[0];
            var rootFile = flow.js.rootNSFile[root];
            var nsToken = getNamespace(namespace);
            var newFile;
            var file = this.file;

            if (!rootFile)
            {
              // TODO: resolve filename relative to html file
              rootFile = flow.files.add({
                //isBasisModule: true,
                filename: path.resolve(flow.js.rootBaseURI[root] || path.dirname(flow.options.file), root + '.js'),  
                nsToken: nsToken, 
                package: 'basis',
                namespace: root,
                isResource: true
              });
              flow.js.rootNSFile[root] = rootFile;
              flow.js.fn2ns[rootFile.jsRef] = root;
              token.file = rootFile;
            }

            if (root == namespace)
            {
              newFile = rootFile;
            }
            else
            {
              newFile = flow.files.add({
                filename: rootFile.resolve(parts.join('/') + '.js'),
                nsToken: nsToken
              });
              newFile.isResource = true;
              newFile.namespace = namespace;

              flow.js.fn2ns[newFile.jsRef] = namespace;
              token.file = newFile;
              //newFile.package = root;
            }

            //token.file = newFile;

            createScope(newFile, flow, nsToken);

            file.link(newFile, token);
            file.deps.push(newFile);

            token.resourceRef = newFile;
            flow.js.resources.push(token);

            fconsole.incDeep(2);
            processFile(newFile, flow);
            fconsole.decDeep(2);
          }
          else
          {
            var arg1 = args[1] && args[1][0] == 'string' ? args[1][1] : '';  // TODO: check it
            var filename = path.resolve(path.dirname(flow.options.file), arg1, arg0); // TODO: resolve filename relative to html file
            var file = this.file;

            var newFile = flow.files.add({
              filename: filename,
              jsRefCount: 0
            });

            newFile.isResource = true;
            newFile.jsRefCount++;

            file.link(newFile, token);
            file.resources.push(newFile);

            token.resourceRef = newFile;
            flow.js.resources.push(token);

            if (newFile.type == 'script')
            {
              createScope(newFile, flow);

              fconsole.incDeep(2);
              processFile(newFile, flow);
              fconsole.decDeep(2);

              token.ref_ = newFile.jsScope.get('module').token.obj.exports;
              token.obj = token.ref_.obj;
            }
            else
            {
              token.ref_ = ['string', newFile.content];
            }
          }
        }
      }
    };

    for (var key in handlers)
      defineHandler(basisScope, key, handlers[key]);

    // search for config and populate it
    var basisConfig = file.jsScope.resolve(['name', 'config']);
    if (basisConfig)
    {
      fconsole.start('[i] config token found');

      basisConfig.obj = {};

      // if (file.autoload)
      // {
      //   fconsole.log('  * add autoload to config: ' + file.autoload);
      //   basisConfig.obj.autoload = ['string', file.autoload];
      // }

      fconsole.end();
    }
    else
    {
      fconsole.log('[!] config token not found');
    }

    // basis.Class.create
    // var classRunner = require('./class.js').runner(flow);
    // try {
    //   basisScope.get('Class').token[1][3][10][1][2][1][1][3][1].run = classRunner;
    // } catch(e) {
    //   defineHandler(globalScope, 'basis.Class.create', classRunner);
    // }

    // process ast
    file.ast = at.struct(file.ast, {
      file: file
    });

    //var CLASS_SELF = globalScope.resolve(at.parse('basis.Class.SELF', true));

    // basis.resource
    defineHandler(globalScope, 'basis.resource', getResourceRunner);

    // basis.asset
    defineHandler(globalScope, 'basis.asset', function(token, this_, args){
      //fconsole.log('basis.asset');
      
      args = token[2]; // FIXME: it's a hack
      var newFilename = args[0][0] == 'string' ? args[0][1] : at.getCallArgs(args, this.file.context, flow, this.file)[0];
      if (newFilename)
      {
        var newFile = flow.files.add({
          filename: newFilename
        });
        //file.link(newFile, token);
        newFile.outputContent = newFile.content;
        newFile.outputFilename = flow.outputResourceDir + newFile.digest + newFile.ext;
        newFile.fileRef = newFile.relOutputFilename;

        token.splice(0, token.length, 'string', newFile.fileRef || '');

        return token;
      }
    });

    // autoload here because we can't resolve basis.resource before ast.struct
    if (file.autoload)
    {
      var autoRequire = at.parse('basis.require("' + file.autoload + '")', true);
      //file.ast[1][1][1][1][3].push(autoRequire);
      handlers.requireNamespace.call({ file: file }, autoRequire, null, [['string', file.autoload]]);
      flow.js.autoload = autoRequire;
    }
  }
  else
  {
    // FIXME: temporary here - move to proper place
    if (file.namespace == 'basis.template.htmlfgen')
    {
      flow.tmpl.fgen = file;
    }
    if (file.namespace == 'basis.template')
    {
      flow.tmpl.module = file;
      var getTheme = function(name){
        if (!name)
          name = 'base';

        if (flow.tmpl.themes[name])
          return flow.tmpl.themes[name];

        var fn = function(name){
          return at.createRunner(function(token, this_, args){
            fconsole.log('[basis.template] template#' + name + ' call', args);
          });
        };

        function addSource(key, value){
          fconsole.log('[basis.template] define template `' + key + '` in `' + theme.name + '` theme');
          resources[key] = value;
          if (value[0] == 'call' && value.resourceRef)
            value.themeDefined = true;
          return value;
        }

        var resources = {};
        var theme = {
          name: name,
          fallback: fn('fallback'),
          define: at.createRunner(function(token, this_, args){
            //fconsole.log('define', args);
            if (!args.length || !args[0])
              return flow.warn({
                fatal: true,
                file: this.file.relpath,
                message: ['basis.template.define w/o args', args]
              });

            var what = this.scope.simpleExpression(args[0]);
            var by = args[1] ? this.scope.simpleExpression(args[1]) : null;

            //fconsole.log('define', what, by);
            if (!what && args[0])
              return flow.warn({
                fatal: true,
                file: this.file.relpath,
                message: 'basis.template.define: first parameter is not resolved, token: ' + at.translate(token)
              });

            if (!by && args[1])
              return flow.warn({
                fatal: true,
                file: this.file.relpath,
                message: 'basis.template.define: second parameter is not resolved, token: ' + at.translate(token)
              });

            if (what[0] == 'string')
            {
              if (!by || by[0] != 'object')
              {
                if (!by || args.length == 1)
                {
                  // return getSourceByPath(what);
                }
                else
                {
                  return addSource(what[1], by);
                }
              }
              else
              {
                var namespace = what[1];
                var props = by[1];
                var result = ['object', []];
                result.obj = {};

                for (var i = 0; i < props.length; i++)
                  result.obj[namespace + '.' + props[i][0]] = addSource(namespace + '.' + props[i][0], props[i][1]);

                return result;
              }
            }
            else
            {
              if (what[0] == 'object')
              {
                var props = what[1];

                for (var i = 0; i < props.length; i++)
                  addSource(props[i][0], props[i][1]);

                 return theme;
              }
              else
              {
                flow.warn({
                  file: this.file.relpath,
                  message: 'Wrong first argument for basis.template.Theme#define'
                });
              }
            }
          }),
          apply: fn('apply'),
          getSource: fn('getSource'),
          drop: at.createRunner(function(token, this_, args){
            flow.warn({
              file: this.file.relpath,
              message: 'basis.template.theme#drop should never be called in build'
            });
          })
        };

        flow.tmpl.themes[name] = theme;
        flow.tmpl.themeResources[name] = resources;

        return theme;
      };

      // basis.template.theme
      defineHandler(file.jsScope, 'getTheme', function(token, this_, args){
        //fconsole.log('getTheme');
        var name = args[0] && args[0][0] == 'string' ? args[0][1] : '';
        token.obj = getTheme(name);
      });
    };

    // process ast
    file.ast = at.struct(file.ast, {
      file: file
    });

    if (file.nsToken)
    {
      astComplete(file.jsScope, file.nsToken, file.nsToken.obj.exports);
      //astExtend(file.jsScope, file.nsToken.obj.exports, file.nsToken.obj.exports);
    }

    // FIXME: temporary here - move to proper place
    if (file.namespace == 'basis.l10n')
    {
      flow.l10n.module = file;

      if (file.jsScope.resolve(at.parse('basis.l10n.dictionary', 1)))
      {
        //
        // NEW API
        //

        flow.l10n = l10nContext.create(flow);
        flow.l10n.module = file;

        var resolveL10nToken = function(key, dictPath){
          var id = key + '@' + dictPath;
          var tokenDescriptor = flow.l10n.getToken(id);

          if (!tokenDescriptor.jsToken)
          {
            var token = ['object', []];
            token.objSource = {};
            token.obj = {
              compute: at.createRunner(function(token_, this_, args){
                var id = key + '.{?}' + '@' + dictPath;

                fconsole.log('[basis.l10n] basis.l10n.Dictionary#token.compute ' + id);
                token_.obj = resolveL10nToken(key + '.{?}', dictPath).jsToken.obj;
              })
            };

            tokenDescriptor.jsToken = token;
          }

          return tokenDescriptor;
        };

        var resolveDictionary = function(filename){
          var dict = flow.l10n.getDictionary(filename);

          if (!dict.jsToken)
          {
            var file = dict.file;
            var token = ['object', []];
            token.file = file;
            token.objSource = {};
            token.obj = {
              token: at.createRunner(function(token_, this_, args){
                var key = this.scope.simpleExpression(args[0]);

                if (key && key[0] == 'string')
                {
                  var id = key[1] + '@' + file.relpath;
                  fconsole.log('[basis.l10n] basis.l10n.Dictionary#token ' + id);

                  var l10nToken = resolveL10nToken(key[1], file.relpath);
                  l10nToken.addRef(this.file, token_);
                  token_.obj = l10nToken.jsToken.obj;
                }
                else
                {
                  flow.warn({
                    file: this.file.relpath,
                    message: 'basis.l10n.dictionary#token: first argument is not resolved, token: ' + at.translate(token_)
                  });
                }
              })
            };

            dict.jsToken = token;
          }

          return dict;
        }; 

        // TODO: fetch culture list from basis.l10n
        defineHandler(globalScope, 'basis.l10n.setCultureList', function(token, this_, args){
          // FIXME: make better resolve solution
          var list = at.getCallArgs(args, this.file.context, flow, this.file)[0];
          flow.l10n.cultures = {};

          fconsole.start('[basis.l10n] ' + at.translate(token) + ' in ' + this.file.relpath);

          if (typeof list == 'string')
            list = list.trim().split(/\s+/);

          if (Array.isArray(list))
          {
            for (var i = 0, cultureDef; cultureDef = list[i]; i++)
            {
              var clist = cultureDef.split(/\//);
              list[i] = clist[0];
              flow.l10n.cultures[clist[0]] = {};
            }

            fconsole.log('[OK] Set culture list ' + JSON.stringify(Object.keys(flow.l10n.cultures)));
          }
          else
          {
            flow.warn({
              file: this.file.relpath,
              message: 'basis.l10n.setCultureList is not resolved (can\'t convert into array): ' + at.translate(token)
            });
          }

          fconsole.end();
        });

        defineHandler(file.jsScope, 'basis.l10n.dictionary', function(token, this_, args){
          var filename = this.scope.simpleExpression(args[0]);
          if (filename && filename[0] == 'string')
          {
            filename = this.file.resolve(filename[1]);
            var dict = resolveDictionary(filename);
            dict.addRef(this.file, token);
            dict.file.jsRefCount++;

            fconsole.log('[basis.l10n] dictionary ' + at.translate(args[0]) + ' -> ' + filename + ' -> ' + dict.file.relpath);            

            token.obj = dict.jsToken.obj;
            this.file.link(dict.file);
          }
          else
          {
            return flow.warn({
              //fatal: true,
              file: this.file.relpath,
              message: 'basis.l10n.dictionary: first argument is not resolved, token: ' + at.translate(token)
            });
          }
        });

        /*defineHandler(file.jsScope, 'basis.l10n.token', function(token, this_, args){
          var expr = args[0];

          var key = this.scope.simpleExpression(expr);
          if (key && key[0] == 'string')
          {
            fconsole.log('[basis.l10n] basis.l10n.token ' + key[1]);

            var entry = {
              token: token,
              key: key[1],
              file: this.file
            };
            token.l10n = entry;
            getTokenList.push(entry);
          }
          else
          {
            // TODO: temporary solution, fix me
            if (!/^basis\.(ui|template|template\.html)$/.test(this.file.namespace))
            {
              flow.warn({
                file: this.file.relpath,
                message: 'basis.l10n.getToken is not resolved: ' + at.translate(token)
              });
            }
          }
        });*/
      }
      else
      {
        //
        // OLD API
        //

        var defList = flow.l10n.defList;
        var getTokenList = flow.l10n.getTokenList;
        var pathes = flow.l10n.pathes;
        var cultureList = flow.l10n.cultureList;

        // TODO: fetch culture list from basis.l10n
        defineHandler(globalScope, 'basis.l10n.setCultureList', function(token, this_, args){
          // FIXME: make better resolve solution
          var list = at.getCallArgs(args, this.file.context, flow, this.file)[0];

          fconsole.start('[basis.l10n] ' + at.translate(token) + ' in ' + this.file.relpath);

          if (typeof list == 'string')
            list = list.trim().split(/\s+/);

          if (Array.isArray(list))
          {
            for (var i = 0, cultureDef; cultureDef = list[i]; i++)
            {
              var clist = cultureDef.split(/\//);
              list[i] = clist[0];
            }

            fconsole.log('[OK] Set culture list ' + JSON.stringify(list));
            list.forEach(cultureList.add, cultureList);
          }
          else
          {
            flow.warn({
              file: this.file.relpath,
              message: 'basis.l10n.setCultureList is not resolved (can\'t convert into array): ' + at.translate(token)
            });
          }

          fconsole.end();
        });        

        // basis.l10n.createDictionary
        defineHandler(file.jsScope, 'basis.l10n.createDictionary', function(token, this_, args){
          //fconsole.log('basis.l10n.createDictionary', args);

          // TODO: remove this kostil'
          args = args.map(this.scope.simpleExpression, this.scope);
          var name = args[0] && args[0][0] == 'string' ? args[0][1] : null;
          var dir = args[1] && args[1][0] == 'string' ? args[1][1] : null;
          var keys = args[2] && args[2][0] == 'object' ? args[2].obj : null;

          if (!name)
            return flow.warn({
              fatal: true,
              file: this.file.relpath,
              message: 'basis.l10n.createDictionary: first parameter is not resolved, token: ' + at.translate(token)
            });

          if (!dir)
            return flow.warn({
              fatal: true,
              file: this.file.relpath,
              message: 'basis.l10n.createDictionary: second parameter is not resolved, token: ' + at.translate(token)
            });

          if (!keys)
            return flow.warn({
              fatal: true,
              file: this.file.relpath,
              message: 'basis.l10n.createDictionary: third parameter is not resolved or not an object, token: ' + at.translate(token)
            });

          var keysObj = {};
          for (var key in keys)
          {
            var val = this.scope.simpleExpression(keys[key]);

            if (!val || val[0] != 'string')
              return flow.warn({
                fatal: true,
                file: this.file.relpath,
                message: 'basis.l10n.createDictionary: value is not resolved (third parameter), value: ' + at.translate(val) + ', token: ' + at.translate(token)
              });

            keysObj[key] = val[1];
          }

          var entry = {
            args: token[2],
            name: name,
            path: path.resolve(flow.options.base, dir),
            keys: keysObj,
            file: this.file
          };

          fconsole.log('[basis.l10n] createDictionary ' + entry.name + ' -> ' + reldir(flow, entry.path));

          token.l10n = entry;
          defList.push(entry);

          if (!pathes[entry.path])
            pathes[entry.path] = {
              __files: []
            };

          pathes[entry.path].__files.add(this.file);
          pathes[entry.path][entry.name] = this.file;
        });

        // basis.l10n.getToken
        defineHandler(file.jsScope, 'basis.l10n.getToken', function(token, this_, args){
          //fconsole.log('basis.l10n.getToken', args);
          
          var expr = args[0];
          for (var i = 1, arg; arg = args[i]; i++)
            expr = ['binary', '+', expr, ['binary', '+', ['string', '.'], arg]];

          var key = this.scope.simpleExpression(expr);
          if (key && key[0] == 'string')
          {
            fconsole.log('[basis.l10n] getToken ' + key[1]);

            var entry = {
              token: token,
              key: key[1],
              file: this.file
            };
            token.l10n = entry;
            getTokenList.push(entry);
          }
          else
          {
            // TODO: temporary solution, fix me
            if (!/^basis\.(ui|template|template\.html)$/.test(this.file.namespace))
            {
              flow.warn({
                file: this.file.relpath,
                message: 'basis.l10n.getToken is not resolved: ' + at.translate(token)
              });
            }
          }
        });
      }
    }
  }
}

function reldir(flow, dir){
  return path.relative(flow.options.base, dir).replace(/\\/g, '/') + '/';  // [base]
}

function processBasisFile(flow, file, attrValue){
  var fconsole = flow.console;

  file.basisScript = true;
  file.basisConfig = attrValue;
  file.config = {
    path: { basis: '' }
  };
  file.namespace = 'basis';
  file.package = 'basis';

  flow.js.rootNSFile.basis = file;
  flow.js.basisScript = file.filename;

  //
  // parse basis config
  //

  fconsole.start('    Parse config:');

  if (/\S/.test(file.basisConfig))
  {
    var config = {};
    try {
      config = Function('return{' + file.basisConfig + '}')();
    } catch(e) {
      fconsole.log('      [WARN] basis-config parse fault: ' + e);
    }

    if (!config.path)
      config.path = {};

    for (var property in config)
    {
      switch (property) {
        case 'path':
          for (var key in config.path)
          {
            flow.js.rootBaseURI[key] = file.htmlFile.resolve(config.path[key]) + '/';
            file.config.path[key] = '';
            fconsole.log('    * Path found for `' + key + '`: ' + config.path[key] + ' -> ' + flow.js.rootBaseURI[key]);
          }
          break;

        case 'noConflict':
        case 'extProto':
          file.config[property] = !!config[property];
          break;

        case 'autoload':
          if (config.autoload)
          {
            fconsole.log('    * Autoload found: ' + config.autoload);

            var autoload = config.autoload;
            var m = config.autoload.match(/^((?:[^\/]*\/)*)([a-z$_][a-z0-9$_]*)((?:\.[a-z$_][a-z0-9$_]*)*)$/i);
            var rootNS;
            if (m)
            {
              if (m[2] != 'basis')
              {
                rootNS = m[2];
                autoload = m[2] + (m[3] || '');
                fconsole.log('      [i] namespace: ' + autoload);
                if (m[1])
                {
                  fconsole.log('      [i] set path for `' + m[2] + '`' + (m[2] in flow.js.rootBaseURI ? ' (override)' : '') + ': ' + m[1]);
                  flow.js.rootBaseURI[m[2]] = file.htmlFile.resolve(m[1]) + '/';
                  file.config.path[rootNS] = '';
                }
              }
              else
              {
                autoload = false;
                ;;;fconsole.log('      [!] value for autoload can\'t be `basis` (setting ignored)');
              }
            }
            else
            {
              autoload = false;        
              ;;;fconsole.log('      [!] wrong autoload value (setting ignored)');
            }

            if (autoload)
            {
              fconsole.log('      [i] full path: ' + (flow.js.rootBaseURI[rootNS] || file.htmlFile.baseURI) + autoload.replace(/\./g, '/') + '.js');
              file.autoload = autoload;
            }
          }
          break;

        default:
          // ignore
      }
    }
  }
  else
    fconsole.log('    <config is empty>');

  fconsole.end();
}
